unit ImageManager;

interface
uses SysUtils, SDL, SDL_image;

var RaiseOnError : Boolean;

function LoadImage (name : String) : PSDL_surface;

type EImageManager = class (Exception);

implementation

type
   PNode = ^TNode;
   TNode = record
      elem : PSDL_surface;
      tail : PNode
   end;
   
var
   First, Last : PNode;

function LoadImage (name : String) : PSDL_surface;
begin
   try
      Last^.elem := IMG_Load (PChar(name));
      LoadImage := Last^.elem;
      new (Last^.tail);
      Last := Last^.tail;
      Last^.elem := nil; Last^.tail := nil
   except
      if Last^.elem <> nil then
         SDL_FreeSurface (First^.elem);
      Last^.elem := nil;
      LoadImage := nil;
      if RaiseOnError then
         raise EImageManager.Create ('Couldn''t load ' + name +
                                        ' -- ' + SDL_GetError);
   end
end;
   
initialization
   RaiseOnError := True;
   new (first);
   First^.elem := nil;
   First^.tail := nil;
   Last := First
      
finalization
   {Uses Last as temporary variable.}
   repeat
      if First^.elem <> nil then SDL_FreeSurface (First^.elem);
      Last := First; 
      First := First^.tail;
      dispose (Last)
   until First = nil
end.
      